
/*
Copyright (c) 2010 Donatien Garnier (donatiengar [at] gmail [dot] com)
 
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef HTTP_SERVER_H
#define HTTP_SERVER_H

class HttpRequestHandler;
class HttpRequestDispatcher;

#include "if/net/net.h"
#include "HttpRequestHandler.h"
#include "HttpRequestDispatcher.h"

#include <string>
using std::string;

#include <map>
using std::map;

class HttpServer
{
public:
  HttpServer();
  ~HttpServer();

  template<typename T>
  void addHandler(const char* path) //Template decl in header
  { m_lpHandlers[path] = &T::inst; }
  
  void bind(int port = 80);
  
private:
  friend class HttpRequestDispatcher;

  void onTcpSocketEvent(TcpSocketEvent e);
  
  TcpSocket* m_pTcpSocket;
  map< string, HttpRequestHandler*(*)(const char*, const char*, TcpSocket*) > m_lpHandlers;

};

//Including handlers here for more convenience
#include "impl/RpcHandler.h"
#include "impl/FSHandler.h"
#include "impl/SimpleHandler.h"

#endif
